﻿import LuminicBox.Log.Logger;
//import LuminicBox.Log.Level;
import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.TracePublisher;

/**
* Class: com.kelvinluck.util.LogWrapper
* 
* Class to wrap the LuminicBox.Log logging system from 
* <http://www.luminicbox.com/blog/?page=post&id=2>
* 
* I started out developing using Colin Moock's org.moock.logger.Logger logging
* class then discovered this cooler one. I thought that rather than changing all my calls 
* to the logger class I could just wrap the LumicBox class in this class of my own.
* 
* The class implements the Singleton pattern and provides access to the methods of 
* LuminicBox.Log that I have needed...
* 
* Especially useful when used in conjection with a <SOSLogPublisher> and MTASC!
* 
* Version:
* 0.1.0
* 
* See Also:
* <SOSLogPublisher>
* 
* <LuminicBox.Log.ConsolePublisher>
* 
* <LuminicBox.Log.TracePublisher>
* 
* <http://kelvinluck.com/article/logging-from-fame>
* 
* Author:
* Kelvin Luck <kelvin@kelvinluck.com> (but it just wraps the LuminicBox code which I didn't write)
**/

class com.kelvinluck.util.LogWrapper {
	
	/**
	* Variable: logCallingFile
	* When used in conjunction with MTASC's TRACE functionality this
	* flags whether to output information about the file and class that 
	* the error was thrown at.
	*/
	static public var logCallingFile:Boolean = true;
	
	/**
	* Variable: DEBUG
	* An integer representing a log level of DEBUG
	**/
	static public var DEBUG:Number = 2;
	/**
	* Variable: INFO
	* An integer representing a log level of INFO
	**/
	static public var INFO:Number = 4;
	/**
	* Variable: WARN
	* An integer representing a log level of WARN
	**/
	static public var WARN:Number = 8;
	/**
	* Variable: ERROR
	* An integer representing a log level of ERROR
	**/
	static public var ERROR:Number = 16;
	/**
	* Variable: FATAL
	* An integer representing a log level of FATAL
	**/
	static public var FATAL:Number = 32;
	
	static var _instance:LogWrapper;
	private var _log:Logger;
	
	/**
	* Function: LogWrapper
	* Don't call the constructor directly, instead use <LogWrapper.getInstance>
	**/
	private function LogWrapper()
	{
	}
	
	/**
	* Function: mtascTrace
	* Function to be used in conjunction with the MTASC TRACE
	* command. This has the advantage of adding className, fileName and
	* lineNumber to your log if <logCallingFile> == true
	* 
	* To use add the following parameter to your mtasc commandline:
	* (start code)
	* -trace com.kelvinluck.util.LogWrapper.mtascTrace
	* (end)
	**/
	static function mtascTrace(message:Object, level:Number, className:String, fileName:String, lineNumber:Number)
	{
		var funcName:String;
		switch (level) {
			case FATAL:
				funcName = "fatal";
				break;
			case ERROR:
				funcName = "error";
				break;
			case WARN:
				funcName = "warn";
				break;
			case INFO:
				funcName = "info";
				break;
			case DEBUG:
				funcName = "debug";
				break;
			default:
				// no level was passed so need to switch the arguments.
				lineNumber = Number(fileName);
				fileName = className;
				className = String(level);
				funcName = "debug";
		}
		if (logCallingFile) {
			getLog()[funcName](className + " ["+fileName+":"+lineNumber+"]");
		}
		getLog()[funcName](message);
	}
	/**
	* Function: init
	* Initalises the <LogWrapper>
	**/
	public function init (logId:String):Void
	{
		// create logger instance
		_log = new Logger(logId);
	}
	
	/**
	* Function: addConsolePublisher
	* 
	* Call this function if you want logged output to be displayed in the seperate 
	* "console" swf.
	* 
	* See Also:
	* <LuminicBox.Log.ConsolePublisher>
	**/
	public function addConsolePublisher()
	{
		_log.addPublisher(new ConsolePublisher());
	}
	/**
	* Function: addTracePublisher
	* 
	* Call this function if you want logged output to be displayed in the trace window of
	* the Flash IDE.
	* 
	* See Also:
	* <LuminicBox.Log.TracePublisher>
	**/
	public function addTracePublisher()
	{
		_log.addPublisher(new TracePublisher());
	}
	/**
	* Function: getInstance
	* 
	* Returns a reference to the Single LogWrapper instance.
	* 
	* Returns:
	* LogWrapper	-	The LogWrapper instance
	**/
	public static function getInstance():LogWrapper
	{
		if (_instance == undefined) {
			_instance = new LogWrapper();
		}
		return _instance;
	}
	/**
	* Function: getLog
	* 
	* Returns a reference to the LuminicBox.Log.Logger instance that is being
	* wrapped.
	* 
	* Returns:
	* Logger		-	A reference to the <LuminicBox.Log.Logger> instance that is being wrapped
	* 
	* See Also:
	* <LuminicBox.Log.Logger>
	**/
	public static function getLog():Logger
	{
		return getInstance()._log;
	}
	
	public function toString():String
	{
		return "[Object - com.kelvinluck.util.LogWrapper]";
	}
	
}